#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/delay.h>

#include "onewire.h"
#include "ds18x20.h"

#define MAXSENSORS 1


uint8_t gSensorIDs[MAXSENSORS][OW_ROMCODE_SIZE];

uint8_t search_sensors(void)
{
	uint8_t i;
	uint8_t id[OW_ROMCODE_SIZE];
	uint8_t diff, nSensors;
	
	nSensors = 0;
	
	for( diff = OW_SEARCH_FIRST; 
		diff != OW_LAST_DEVICE && nSensors < MAXSENSORS ; )
	{
		DS18X20_find_sensor( &diff, &id[0] );
		
		if( diff == OW_PRESENCE_ERR ) {
// No sensor found
			PORTA |= (1 << PA2);
			break;
		}
		
		if( diff == OW_DATA_ERR ) {
//bus error
			PORTA |= (1 << PA4);
			break;
		}
		
		for (i=0;i<OW_ROMCODE_SIZE;i++)
			gSensorIDs[nSensors][i]=id[i];
		
		nSensors++;
	}
	
	return nSensors;
}


void enable_pwm()
{
	DDRA |= (1<<PA6);
	//Anschalten des PWM an OC1A 
	TCCR1A |= (1<<COM1A1);
	TCCR1A |= (1<<WGM11);
	TCCR1B |= (1<<WGM12);
	// den Prescaler des Counters auf 1024 setzen
	// Dieser Aufruf beeinflusst das Auslesen des DS1820 
	// bei einem Prescaler ungleich 0 oder 1 (aus oder Prozessortakt)
	// lese ich nur noch 85°C 
	TCCR1B |= ((1<<CS10)|(1<<CS12));
	OCR1AH = 0x00;
	OCR1AL = 0x00;
}


uint8_t readSwitch()
{
	if( !(PINB & (1<<PB2)) )
	{
		_delay_ms(100);
		if( !(PINB & (1<<PB2)) )
		{
			_delay_ms(100);
			return 1; 
		}
	}
	return 0;
}



int main()
{
  uint8_t i;
  uint8_t subzero, cel_frac_bits, flag;
  uint8_t cel = 0;
  uint8_t lcel = 0; 
  uint8_t hcel = 0; 

  uint16_t temp; 

  DDRB  &= ~(1<<PB2); // PORTB als iEingang schalten
  PORTB |= (1 << PB2); //PUllup an für Taster

  DDRA |= (1 << PA5 )|(1 << PA6)|(1 << PA2 )|(1 << PA4)|(1 << PA3); 
  PORTA &= ~((1 << PA2 )|(1 << PA3)|(1 << PA4)|(1 << PA5));

  sei();

  enable_pwm();

  i = search_sensors();
	if( i == 1 )
	{
		PORTA &= ~((1 << PA2 )|(1 << PA3)|(1 << PA4)|(1 << PA5));
		//ein Lämpchen an machen, wenn ein Sensor gefunden wurde. 
		PORTA |= (1<<PA4);
	}
	

  for(;;)
  {

	
	//Temperatur fühlen
	i = gSensorIDs[0][0];
	flag = DS18X20_start_meas(DS18X20_POWER_EXTERN, NULL);
	if(flag == DS18X20_OK)
	{
		_delay_ms(DS18B20_TCONV_12BIT);
		flag = DS18X20_read_meas_single(i, &subzero, &cel, &cel_frac_bits);

		if( flag == DS18X20_OK )
		{

			//ich habe an der Schaltung nur vier LED, daher aufteilen der 8-bit Temperatur auf zwei mal 4-Bit. 
			lcel = cel; 
			hcel = cel; 
	
			lcel = lcel & ((1<<3)|(1<<2)|(1<<1)|(1));
			hcel = (hcel >> 4);


			while(1)
			{
				if(readSwitch()) break; 
			}

			//ausgeben der 4 oberen bits der gemessenen temp
			PORTA &= ~((1 << PA2 )|(1 << PA3)|(1 << PA4)|(1 << PA5));
			PORTA |= (hcel << PA2);

			while(1)
			{
				if(readSwitch()) break; 
			}
			//ausgeben der 4 unteren bits der gemessenen temp
			PORTA &= ~((1 << PA2 )|(1 << PA3)|(1 << PA4)|(1 << PA5));
			PORTA |= (lcel << PA2);
		
			while(1)
			{
				if(readSwitch()) break; 
			}
			
			//mal alle anmachen, damit ich weiss, dass gleich neu gemessen wird.
			PORTA |= ((1 << PA2 )|(1 << PA3)|(1 << PA4)|(1 << PA5));
		}
		else
		{
			//fehler als rote Lampe melden
			PORTA &= ~((1 << PA2 )|(1 << PA3)|(1 << PA4)|(1 << PA5));
			PORTA |= (1<<PA2);
		}
	}
	else
	{
		//fehler als rote Lampe melden
		PORTA &= ~((1 << PA2 )|(1 << PA3)|(1 << PA4)|(1 << PA5));
			PORTA |= (1<<PA2);
	}
  } //ever
}

